% El comando predeterminado que trae Octave para resolver PVIs es lsode 
%(Ojo, en el caso de Matlab este emplea comandos del tipo odexx, disponibles en Octave en el paquete opcional odepkg)
help lsode
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Ejemplo de resolución modelos presa-depredador
% u_t = au -b u v    v_t = -cv + d u v
%
function xdot=func(x,t)
a=1;b=1;c=1;d=1;
xdot(1,1)=a*x(1)-b*x(1)*x(2);
xdot(2,1)=-c*x(2)+ d*x(1)*x(2);
end
t=linspace(0,10,100);
% Convergencia a la solución periódica
sol=lsode("func",[0.5;0.5],t);
plot(t,sol(:,1),t,sol(:,2))
% Alternativamente
plot(t,sol')
plot(sol(:,1),sol(:,2))
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Ejemplo de resolución de leyes de acción de masas
%  A \to(K1) B \to(K2) C
%  
function ddot=func(d,t)
k1=5; k2=1;
ddot=[-k1,0,0;k1,-k2,0;0,k2,0]*d;
end
t=linspace(0,10,100);
% Convergencia a la solución estacionaria
sol=lsode("func",[0.5;0.5;0],t);
plot(t,sol)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Ejemplo de resolución de leyes de acción de masas
%  A + 2 B \xrightleftharpoons[k_{-1}]{k_{1}}  C
%  
function ddot=func(d,t)
K1=10; Kmenos1=1;
ddot(1,1)=- K1*d(1)*(d(2)^2)+Kmenos1*d(3);
ddot(2,1)=-2*K1.*d(1)*(d(2)^2)+2*Kmenos1*d(3);
ddot(3,1)=K1*d(1)*(d(2)^2) - Kmenos1*d(3);
end
t=linspace(0,10,500);
% Convergencia a la solución estacionaria
sol=lsode("func",[0.5;0.5;0],t);
plot(t,sol)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Opciones de lsode
lsode_options
lsode_options('integration method','non-stiff')
% Al cambiar el método de integración podemos ver que la simulacion va a ser más rápida
% Para ver el funcionamiento concreto del comando con cada una de estas opciones ver el manual de Octave.
% Para detallar el funcionamiento del comando: Alan C. Hindmarsh, ODEPACK, A Systematized Collection of ODE Solvers, in Scientific Computing, R. S. Stepleman, editor, (1983)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% No obstante, podemos programar nuestros propios métodos sin problema alguno. Ejemplo Euler Explícito:
%

clf;
function [t,y]=euler(f,t0,tf,u0,m)
% Función que implementa el método de Euler explícito de orden 1. Datos:
% f la función que define la ecuación diferencial,
% t0 y tf instantes inicial y final de la simulación,
% u0 condición inicial (vector columna)
% 
  k= (tf-t0)/(m);          
  t=linspace(t0,tf,m+1);                       
  y=u0;                    
   for i= 1:m	
	k1 = feval(f,y(:,i),t(i));
	y=[y, y(:,i) + k*k1];
  end;
  y=y';
end;
[t,y]=euler('func',0,10,[0.5;0.5;0],100);
plot(t,y(:,1),t,y(:,2),t,y(:,3))
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Ejemplo: resolución de la ecuación de difusión con evolución temporal.
% Aplica el método de lineas al problema de evolución
% u_t = \nu u_{xx} -k u , x\in[a,b] t \in [0,T],  u(a,t)= \alpha, u (b,t)= \beta 
% Discretiza en espacio mediante una aproximación por diferencias finitas centradas.
% Este modelo correspondería a la solucion de un proceso difusivo en el que simultaneamente hay una 
% degradación/mortandad.
%



 % Datos de la ecuación e intervalo espacial
global a b alpha beta nu k m h A rhs
a = 0; b = 2;  
alpha = 1; beta =0;
nu = 0.1;  k = 0.5;
% datos de la discretización
m = 100;                   % numero de nodos interiores
h = (b-a)/(m+1);

% Defino la funcion que determina el sistema de EDOs a integrar temporalmente
e = ones(m,1);
A = spdiags([e -2*e e],[-1 0 1],m,m);			
A= nu*A/h^2;
I = speye(m);
A= A-k*I;
% definimos el termino independiente que tiene en cuenta 
% los valores en la frontera.
rhs=zeros(m,1);
rhs(1) = nu*alpha/h^2;
rhs(m) = nu*beta/h^2; 
 

function ddot=lineas(d,t)
global a b alpha beta nu k m h A rhs
ddot= A*d+rhs;
endfunction
 
% Determinamos los instantes de tiempo en los que queremos ver la solución
T=5;
k= 6 ; %  Número de intervalo de representación temporal 
t=linspace(0,T,k);

% dato inicial para la resolución          
x=linspace(a,b,m+2);
xint=x(2:m+1);
u=cos(pi*xint/4)';
plot(xint,u)         

% Resolvemos numéricamente y representamos la solución
tic;
sol=lsode("lineas",u,t);
toc
usoln=[alpha*ones(k,1),sol,beta*ones(k,1)];
plot(x,usoln)
% Si queremos verlo paso a paso:
clf
hold on
for i=0:k-1
plot(x,usoln(i+1,:),pause(2));
endfor
hold off
% Si queremos hacer un video:
clf
hold off
for i=0:k
plot(x,usoln(i+1,:));
print(["imagen" num2str(k)".png"],'-dpng') 
endfor
%  y luego desde el sistema tendríamos que ejecutar el siguiente comando
% ffmpeg -f image2 -i imagen%01d.png  output2.mpeg
% para montar la película. GRACIAS RAFAEL!!!



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Ejemplo resolución del atractor de Lorentz
function xdot=func(x,t)
a=10;b=28;c=8/3;
xdot(1,1)=a*(x(2)-x(1));
xdot(2,1)=x(1)*(b-x(3))-x(2);
xdot(3,1)=x(1)*x(2)-c*x(3);
end
x0=[1;1;1];
t=linspace(0,10,5000);
tic; x=lsode("func",x0,t);toc
plot3(x(:,1),x(:,2),x(:,3))  % comando para representar curvas paramétricas.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Información sobre cómo resolver otro tipo de ecuaciones diferenciales con esquemas en diferencias finitas.
%
% http://www.amath.washington.edu/~rjl/fdmbook/
%
% Ejemplo 2: Ecuación del calor en 1D
% Ver archivo heat_CM.m
% Ejemplo 3: Ecuación de Laplace en 2D
% Ver archivo poisson.m
%Con un pequeño retoque a la representación gráfica podemos emplearlo
sin problema en Octave
% Nota: Implementaciones de FEM en el paquete bim: Octave en http://wiki.octave.org/Bim_package
%

